@@ -13,11 +13,11 @@ Class::Simple::Cached - cache messages to an object
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 SYNOPSIS
 
@@ -83,11 +83,19 @@ sub _caller_class
 sub AUTOLOAD {
 	our $AUTOLOAD;
 	my $param = $AUTOLOAD;
-
 	$param =~ s/.*:://;
 
-	return if($param eq 'DESTROY');
 	my $self = shift;
+	my $cache = $self->{'cache'};
+
+	if($param eq 'DESTROY') {
+		if(defined($^V) && ($^V ge 'v5.14.0')) {
+			return if ${^GLOBAL_PHASE} eq 'DESTRUCT';	# >= 5.14.0 only
+		}
+		$cache->clear();
+		return;
+	}
+
 	# my $func = $self->{'object'} . "::$param";
 	my $func = $param;
 	my $object = $self->{'object'};
@@ -97,14 +105,15 @@ sub AUTOLOAD {
 		# return $object->$func(\@_);
 	# }
 
-	my $cache = $self->{'cache'};
-
 	if(scalar(@_) == 0) {
 		# Retrieving a value
 		if(my $rc = $cache->get($param)) {
 			if(ref($rc) eq 'ARRAY') {
 				return @{$rc};
 			}
+			if($rc eq __PACKAGE__ . ">UNDEF<") {
+				return;
+			}
 			return $rc;
 		}
 		if(wantarray) {
@@ -115,13 +124,19 @@ sub AUTOLOAD {
 			$cache->set($param, \@rc, 'never');
 			return @rc;
 		}
-		return $cache->set($param, $object->$func(), 'never');
+		my $rc = $object->$func();
+		if(!defined($rc)) {
+			$cache->set($param, __PACKAGE__ . ">UNDEF<", 'never');
+			return;
+		}
+		return $cache->set($param, $rc, 'never');
 	}
 
 	# $param = "SUPER::$param";
 	# return $cache->set($param, $self->$param(@_), 'never');
 	if($_[1]) {
 		# Storing an array
+		# We store a ref to the array, and dereference on retrieval
 		my $val = $object->$func(\@_);
 		$cache->set($param, $val, 'never');
 		return @{$val};
@@ -136,6 +151,8 @@ Nigel Horne, C<< <njh at bandsman.co.uk> >>
 
 =head1 BUGS
 
+Doesn't work with L<Memoize>.
+
 Please report any bugs or feature requests to C<bug-class-simple-cached at rt.cpan.org>,
 or through the web interface at
 L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Class-Simple-Cached>.
